package com.ccwangluo.ccim;

import android.Manifest;

import androidx.fragment.app.FragmentActivity;

import com.ccwangluo.ccim.data.Contact;
import com.ccwangluo.ccim.data.ContactGroup;
import com.ccwangluo.ccim.data.ContactRequestsRes;
import com.ccwangluo.ccim.data.NetStatus;
import com.ccwangluo.ccim.data.PullContactsRes;
import com.ccwangluo.ccim.data.PushChannelMsgRes;
import com.ccwangluo.ccim.data.PushPrivateMsgRes;
import com.ccwangluo.ccim.data.SearchRes;
import com.ccwangluo.ccim.data.Status;
import com.ccwangluo.ccim.data.User;
import com.ccwangluo.ccim.listener.ContactsCallBack;
import com.ccwangluo.ccim.listener.OnConnectListener;
import com.ccwangluo.ccim.listener.OnPushMsgListener;
import com.ccwangluo.ccim.listener.ResultCallback;
import com.ccwangluo.ccim.listener.ResultEventCallback;
import com.ccwangluo.ccim.manage.CallBackManager;
import com.ccwangluo.ccim.manage.SocketManageer;
import com.ccwangluo.ccim.util.MediaHelper;
import com.google.gson.Gson;
import com.google.protobuf.util.JsonFormat;
import com.permissionx.guolindev.PermissionX;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.logging.Logger;

import ccim.protocol.Channel;
import ccim.protocol.Chat;
import ccim.protocol.Common;
import ccim.protocol.ContactOuterClass;
import ccim.protocol.Message;
import ccim.protocol.MessageSpec;
import ccim.protocol.Push;

public class CCIMClient {

    private final Logger logger = Logger.getLogger(CCIMClient.class.getName());

    private static SocketManageer socketManageer;
    private static CallBackManager<Chat.ChatResponseMessage> callBackManager;

    private final HashMap<Long, Contact> contacts = new HashMap<>();
    private final HashMap<Long, ContactGroup> contactGroups = new HashMap<>();

    private static boolean isNeedFresh = true;

    private CCIMClient() {
    }

    private static CCIMClient mInstance;

    public static CCIMClient getInstance() {
        if (mInstance == null) {
            synchronized (CCIMClient.class) {
                if (mInstance == null) {
                    mInstance = new CCIMClient();
                    callBackManager = new CallBackManager<>();
                    socketManageer = new SocketManageer(callBackManager);
                }
            }
        }
        return mInstance;
    }

    public void initSDK(String appKey, String userName, String extData, Long userId, Long ts, String sign, OnConnectListener resultCallback) {

        socketManageer.initChat(appKey, userName, extData, userId, ts, sign, new OnConnectListener() {
            @Override
            public void onSocketConnection() {
                resultCallback.onSocketConnection();
                if (!isNeedFresh) {
                    isNeedFresh = true;
                    contacts.clear();
                    contactGroups.clear();
                    getContactList((contacts, contactGroups) -> {

                    });
                }
            }

            @Override
            public void onSocketDisconnection() {
                logger.info("disconnection");
                resultCallback.onSocketDisconnection();
            }
        });
    }


    public void registerMessageListener(OnPushMsgListener pushMsgListener) {
        callBackManager.addCallBack(0, chatResponseMessage -> {
            Gson gson = new Gson();
            if (chatResponseMessage.hasPushPrivateMsgRes()) {
                MessageSpec.ChatMessage pushPrivateMsgRes = chatResponseMessage.getPushPrivateMsgRes().getMessage();
                pushPrivateMessageAck(pushPrivateMsgRes.getSenderUserId(), pushPrivateMsgRes.getMsgId());
                try {
                    String print = JsonFormat.printer().print(chatResponseMessage.getPushPrivateMsgRes());
                    PushPrivateMsgRes pushPrivateMsgRes1 = gson.fromJson(print, PushPrivateMsgRes.class);
                    pushMsgListener.pushPrivateMsg(pushPrivateMsgRes1);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }

            if (chatResponseMessage.hasPushChannelMsgRes()) {
                Push.PushChannelMessageResponse channelMsgRes = chatResponseMessage.getPushChannelMsgRes();
                pushChannelMessage(channelMsgRes.getChannelId(), channelMsgRes.getMessage().getMsgId());
                try {
                    String print = JsonFormat.printer().print(channelMsgRes);
                    pushMsgListener.pushChannelMsg(gson.fromJson(print, PushChannelMsgRes.class));
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }

            if (chatResponseMessage.hasPushNewContactRes()) {
                Push.PushNewContactResponse pushNewContactRes = chatResponseMessage.getPushNewContactRes();
                try {
                    String print = JsonFormat.printer().print(pushNewContactRes.getContact());
                    Contact contact = gson.fromJson(print, Contact.class);
                    contacts.put(contact.getContactUserId(), contact);
                    pushMsgListener.pushNewContact(contact);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }

            if (chatResponseMessage.hasPushContactDeleteMeRes()) {
                Push.PushContactDeleteMeResponse pushNewContactRes = chatResponseMessage.getPushContactDeleteMeRes();
                long contactUserId = pushNewContactRes.getContactUserId();
                contacts.remove(contactUserId);
                pushMsgListener.pushDeleteContact(contactUserId);
            }

            if (chatResponseMessage.hasPushContactNetStatusRes()) {
                Push.PushContactNetStatusResponse pushNewContactRes = chatResponseMessage.getPushContactNetStatusRes();
                long contactUserId = pushNewContactRes.getContactUserId();
                NetStatus netStatus = NetStatus.valueOf(pushNewContactRes.getNetStatus().name());
                Objects.requireNonNull(contacts.get(contactUserId)).setNetStatus(netStatus);
                pushMsgListener.pushContactNetStatus(contactUserId, netStatus);
            }

            if (chatResponseMessage.hasPushContactNewReqRes()) {
                Push.PushContactNewReqResponse pushContactNewReqRes = chatResponseMessage.getPushContactNewReqRes();
                String username = pushContactNewReqRes.getContactNewReq().getUsername();
                Long id = pushContactNewReqRes.getContactNewReq().getUserId();
                pushMsgListener.pushContactNewReq(new User(id, username));
            }

            if (chatResponseMessage.hasPushBlockedByContactRes()) {
                Push.PushBlockedByContactResponse pushContactNewReqRes = chatResponseMessage.getPushBlockedByContactRes();
                long contactUserId = pushContactNewReqRes.getContactUserId();
                ContactOuterClass.ContactStatus contactStatus = pushContactNewReqRes.getContactStatus();
                Objects.requireNonNull(contacts.get(contactUserId)).setStatus(Status.valueOf(contactStatus.name()));
                pushMsgListener.pushBlockByContact(contactUserId);
            }

        });
    }


    /**
     * 获取联系人列表
     */
    public void getContactList(ContactsCallBack resultEventCallback) {
        if (!isNeedFresh || contacts.size() > 0 || contactGroups.size() > 0) {
            resultEventCallback.onSuccess(contacts.values(), contactGroups.values());
            return;
        }
        Chat.PullContactsRequest builder = Chat.PullContactsRequest.newBuilder()
                .build();
        int id = socketManageer.sendRequest(builder);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasPullContactsRes()) {
                    Chat.PullContactsResponse pullContactsRes = chatResponseMessage.getPullContactsRes();
                    try {
                        String print = JsonFormat.printer().print(pullContactsRes);
                        PullContactsRes pullContactsRes1 = new Gson().fromJson(print, PullContactsRes.class);
                        contacts.clear();
                        contactGroups.clear();
                        if (pullContactsRes1.getContacts() != null){
                            for (Contact contact : pullContactsRes1.getContacts()) {
                                contacts.put(contact.getContactUserId(), contact);
                            }
                        }
                        if (pullContactsRes1.getContactGroups() != null) {
                            for (ContactGroup contactGroup : pullContactsRes1.getContactGroups()) {
                                contactGroups.put(contactGroup.getGroupId(), contactGroup);
                            }
                        }
                        resultEventCallback.onSuccess(contacts.values(), contactGroups.values());
                        isNeedFresh = false;
                    } catch (Exception e) {
                        e.printStackTrace();
                    }

                }
            }

            @Override
            public void onFailed(int errorCode) {
                resultEventCallback.onFailed(errorCode);
            }
        });

    }


    /**
     * 同步离线消息
     */
    private void getOfflineMessage(Long lastPullMsgId, ResultEventCallback<List<Message.PullOfflineMessageResponse.OfflineMessage>> resultEventCallback) {
        Message.PullOfflineMessageRequest offlineMessageRequest = Message.PullOfflineMessageRequest.newBuilder()
                .setLastPulledMsgId(lastPullMsgId)
                .build();
        int id = socketManageer.sendRequest(offlineMessageRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasPullOfflineMsgRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    List<Message.PullOfflineMessageResponse.OfflineMessage> offlineMsgsList = chatResponseMessage.getPullOfflineMsgRes().getOfflineMsgsList();
                    resultEventCallback.onSuccess(offlineMsgsList);
                }
            }

            @Override
            public void onFailed(int errorCode) {
                resultEventCallback.onFailed(errorCode);
            }
        });

    }

    /**
     * 搜索好友
     */
    public void searchUser(String userName, ResultEventCallback<List<User>> resultEventCallback) {
        Chat.SearchRequest searchRequest = Chat.SearchRequest.newBuilder().setUsername(userName).build();
        int id = socketManageer.sendRequest(searchRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasSearchRes()) {
                    Chat.SearchResponse searchRes = chatResponseMessage.getSearchRes();
                    try {
                        String print = JsonFormat.printer().print(searchRes);
                        SearchRes searchRes1 = new Gson().fromJson(print, SearchRes.class);
                        resultEventCallback.onSuccess(searchRes1.getUsers());
                    } catch (Exception e) {
                        e.printStackTrace();
                    }

                }
            }

            @Override
            public void onFailed(int errorCode) {
                resultEventCallback.onFailed(errorCode);
            }
        });
    }

    /**
     * 添加好友
     */
    public void addContact(Long userId, ResultCallback callback) {
        Chat.AddContactRequest addContactRequest = Chat.AddContactRequest.newBuilder().setUserId(userId).build();
        int id = socketManageer.sendRequest(addContactRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasAddContactRes()) {
                    callback.onSuccess();
                }
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });

    }

    /**
     * 联系人申请列表
     */
    public void getContactReqList(ResultEventCallback<List<User>> callback) {
        Chat.ContactRequestsRequest contactRequestsRequest = Chat.ContactRequestsRequest.newBuilder().build();
        int id = socketManageer.sendRequest(contactRequestsRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasContactRequestsRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    Chat.ContactRequestsResponse contactRequestsRes = chatResponseMessage.getContactRequestsRes();
                    try {
                        String print = JsonFormat.printer().print(contactRequestsRes);
                        ContactRequestsRes contactRequestsRes1 = new Gson().fromJson(print, ContactRequestsRes.class);
                        callback.onSuccess(contactRequestsRes1.getRequests());
                    } catch (Exception e) {
                        e.printStackTrace();
                    }

                }
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });

    }

    /**
     * 验证联系人申请
     */
    public void checkContactReq(Long reqUserId, boolean accept, ResultCallback callback) {
        Chat.CheckContactReqRequest checkContactReqRequest = Chat.CheckContactReqRequest.newBuilder()
                .setReqUserId(reqUserId)
                .setAccept(accept)
                .build();
        int id = socketManageer.sendRequest(checkContactReqRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasCheckContactReqRes()) {
                    ContactOuterClass.Contact newContact = chatResponseMessage.getCheckContactReqRes().getNewContact();
                    try {
                        String print = JsonFormat.printer().print(newContact);
                        Contact contact = new Gson().fromJson(print, Contact.class);
                        contacts.put(contact.getContactUserId(),contact);
                        callback.onSuccess();
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }

    /**
     * 创建联系人群组
     */
    public void createContactGroup(String name, ResultEventCallback<Long> callback) {
        Chat.CreateContactGroupRequest createContactGroupRequest = Chat.CreateContactGroupRequest.newBuilder()
                .setName(name)
                .build();
        int id = socketManageer.sendRequest(createContactGroupRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasCreateContactGroupRes()) {
                    Chat.CreateContactGroupResponse createContactGroupRes = chatResponseMessage.getCreateContactGroupRes();
                    long groupId = createContactGroupRes.getGroupId();
                    contactGroups.put(groupId, new ContactGroup(groupId, name));
                    callback.onSuccess(groupId);
                }
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }


    /**
     * 删除联系人群组
     */
    public void deleteContactGroup(Long groupId, ResultCallback callback) {
        ContactOuterClass.DeleteContactGroupRequest deleteContactGroupRequest = ContactOuterClass.DeleteContactGroupRequest.newBuilder().setContactGroupId(groupId).build();
        int id = socketManageer.sendRequest(deleteContactGroupRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                contactGroups.remove(groupId);
                for (Map.Entry<Long, Contact> longContactEntry : contacts.entrySet()) {
                    Contact value = longContactEntry.getValue();
                    if (value.getContactGroupId().equals(groupId)) {
                        value.setContactGroupId(0L);
                    }
                }
                callback.onSuccess();
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }


    /**
     * 移动联系人所在的群组
     */
    public void moveContactGroup(Long groupId, Long userId, ResultCallback callback) {
        Chat.MoveContactGroupRequest moveContactGroupRequest = Chat.MoveContactGroupRequest.newBuilder()
                .setContactGroupId(groupId)
                .setContactUserId(userId)
                .build();
        int id = socketManageer.sendRequest(moveContactGroupRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasMoveContactGroupRes()) {
                    Objects.requireNonNull(contacts.get(userId)).setContactGroupId(groupId);
                    callback.onSuccess();
                }
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }

    /**
     * 删除联系人
     */
    public void deleteContact(Long userId, ResultCallback callback) {
        Chat.DeleteContactRequest deleteContactRequest = Chat.DeleteContactRequest.newBuilder()
                .setContactUserId(userId)
                .build();
        int id = socketManageer.sendRequest(deleteContactRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                contacts.remove(userId);
                callback.onSuccess();
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }

    /**
     * 拉黑联系人
     */
    public void blockContact(Long userId, ResultCallback callback) {
        ContactOuterClass.BlockContactRequest blockContactRequest = ContactOuterClass.BlockContactRequest.newBuilder()
                .setContactUserId(userId)
                .build();
        int id = socketManageer.sendRequest(blockContactRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                Objects.requireNonNull(contacts.get(userId)).setStatus(Status.CONTACT_STATUS_BLACK_LIST);
                callback.onSuccess();
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }

    /**
     * 举报违规用户
     */
    public void reportIllegalUser(String title, String content, Long userId, ResultCallback callback) {
        Chat.ReportIllegalUserRequest reportIllegalUserRequest = Chat.ReportIllegalUserRequest.newBuilder()
                .setContent(content)
                .setIllegalUserId(userId)
                .setTitle(title)
                .build();
        int id = socketManageer.sendRequest(reportIllegalUserRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasReportIllegalUserRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    callback.onSuccess();
                }
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }

    /**
     * 发送消息
     */
    public void sendTextMessage(String content, Long reciveUserId, ResultCallback callback) {
        Contact contact = contacts.get(reciveUserId);
        if (contact == null){
            callback.onFailed(Common.StatusCode.STATUS_CODE_NOT_CONTACT_ERROR.getNumber());
            return;
        }
        if (contact.getStatus().equals(Status.CONTACT_STATUS_BID_BLACKED) || contact.getStatus().equals(Status.CONTACT_STATUS_BLACK_LIST)){
            callback.onFailed(Common.StatusCode.STATUS_CODE_CONTACT_BLOCKED_ME_ERROR.getNumber());
            return;
        }

        if (contact.getStatus().equals(Status.CONTACT_STATUS_BLACKED)){
            callback.onFailed(Common.StatusCode.STATUS_CODE_CONTACT_BLOCKED_ERROR.getNumber());
            return;
        }
        Message.SendMessageRequest messageRequest = Message.SendMessageRequest.newBuilder()
                .setContent(content)
                .setMsgType(MessageSpec.MessageType.MESSAGE_TYPE_TEXT)
                .setReceiverUserId(reciveUserId).build();
        int id = socketManageer.sendRequest(messageRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                callback.onSuccess();
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }


    /**
     * 创建频道
     */
    public void createChannel(String channelName, String tag, boolean isOpen, List<Long> userIds, ResultEventCallback<Long> callback) {
        Channel.CreateChannelRequest.Builder builder = Channel.CreateChannelRequest.newBuilder()
                .setName(channelName)
                .setTag(tag)
                .setOpen(isOpen);
        for (Long id : userIds) {
            builder.addUserIds(id);
        }
        int id = socketManageer.sendRequest(builder.build());
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasCreateChannelRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    Channel.CreateChannelResponse createChannelRes = chatResponseMessage.getCreateChannelRes();
                    callback.onSuccess(createChannelRes.getChannelId());
                }
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }

    /**
     * 加入频道
     */
    public void joinChannel(Long channelId, ResultCallback callback) {
        Channel.JoinChannelRequest.Builder builder = Channel.JoinChannelRequest.newBuilder()
                .setChannelId(channelId);
        int id = socketManageer.sendRequest(builder.build());
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                logger.info(String.valueOf(chatResponseMessage));
                callback.onSuccess();
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }

    /**
     * 移除频道成员
     */
    public void removeChannelMember(Long channelId, Long userId, ResultCallback callback) {
        Channel.RemoveMemberRequest.Builder builder = Channel.RemoveMemberRequest.newBuilder()
                .setChannelId(channelId)
                .setUserId(userId);
        int id = socketManageer.sendRequest(builder.build());
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasRemoveMemberRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    callback.onSuccess();
                }
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }

    /**
     * 退出频道
     */
    public void exitChannel(Long channelId, ResultCallback callback) {
        Channel.ExitChannelRequest.Builder builder = Channel.ExitChannelRequest.newBuilder()
                .setChannelId(channelId);
        int id = socketManageer.sendRequest(builder.build());
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                logger.info(String.valueOf(chatResponseMessage));
                callback.onSuccess();
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }


    /**
     * 发送频道消息
     */
    public void sendChannelMessage(String content, Long channelId, ResultCallback callback) {
        Message.SendChannelMessageRequest sendChannelMessageRequest = Message.SendChannelMessageRequest.newBuilder()
                .setContent(content)
                .setMsgType(MessageSpec.MessageType.MESSAGE_TYPE_TEXT)
                .setChannelId(channelId)
                .build();
        int id = socketManageer.sendRequest(sendChannelMessageRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasSendChannelMsgRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    callback.onSuccess();
                }
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }

    /**
     * 推送私人消息成功ack
     */
    public void pushPrivateMessageAck(Long contactUserId, Long msgId) {
        Push.PushPrivateMessageAckRequest pushPrivateMessageAckRequest = Push.PushPrivateMessageAckRequest.newBuilder()
                .setContactUserId(contactUserId)
                .setMsgId(msgId)
                .build();
        int id = socketManageer.sendRequest(pushPrivateMessageAckRequest);
        callBackManager.addCallBack(id, chatResponseMessage -> logger.info(String.valueOf(chatResponseMessage)));
    }


    /**
     * 推送渠道消息成功ack
     */
    public void pushChannelMessage(Long channelId, Long msgId) {
        Push.PushChannelMessageAckRequest channelMessageAckRequest = Push.PushChannelMessageAckRequest.newBuilder()
                .setChannelId(channelId)
                .setMsgId(msgId)
                .build();
        int id = socketManageer.sendRequest(channelMessageAckRequest);
        callBackManager.addCallBack(id, chatResponseMessage -> logger.info(String.valueOf(chatResponseMessage)));
    }

    /**
     * 开始录音
     */

    public void startRecordAudioMessage(FragmentActivity activity) {
        PermissionX.init(activity)
                .permissions(Manifest.permission.RECORD_AUDIO)
                .onExplainRequestReason((scope, deniedList, beforeRequest) -> scope.showRequestReasonDialog(deniedList, "即将申请的权限是程序必须依赖的权限", "我已明白"))
                .onForwardToSettings((scope, deniedList) -> scope.showForwardToSettingsDialog(deniedList, "您需要去应用程序设置当中手动开启权限", "我已明白"))
                .request((allGranted, grantedList, deniedList) -> {
                    if (allGranted) {
                        MediaHelper.getInstance().prepare(activity);
                    }
                });
    }

    public void stopAndSendAudioMessage() {
        MediaHelper.getInstance().release();
        String filePath = MediaHelper.getInstance().getFilePath();

    }

    public void cancleAudioMessage() {
        MediaHelper.getInstance().cancel();
    }


    public void startPlayAudio(String filePath) {
        MediaHelper.getInstance().play(filePath);
    }

    public void logout() {
        socketManageer.disConnect();
    }
}
